function [cytonum,nascnum,dgs] = simplernasim(R,lambda,tau,tdwell,trec,maxt)

% simulate the cytoplasmic RNA numbers as would be measured by smFISH, to
% see if our ladder of state would be detectible.

% tau is the lifetime of the RNA once it has left the transcription site
% lambda is the initiation rate for each state.  For these simple purposes,
% we assume that polymerases are processive, and so the rate of RNA
% production is equal to lambda.  It will be offset by the dwell time of
% the polymerase.

% because we've got a combination of random events (initiations and decays)
% and pseudo-deterministic (leaving the TS), need to maintain a list of
% event times and add the random event to that list, and move to the next
% event

if nargin<6 || isempty(maxt)
    maxt = 4000;
end
if nargin<5 || isempty(trec)
    trec = 60;
end

% % R2 = [zeros(size(R,1),1),R,zeros(size(R,1),1)];

% in this implementation, evind is a multifunctional reference variable,
% whose interpretation depends on the event which is occurring

% calculate the equilibrium state distribution and randomly sample the
% starting state from it.
Peq = eqprobvect(R,1);
cP = cumsum(Peq);
currstate = sum(cP(1:end-1)<rand) + 1;

cytonum = zeros(floor(maxt/trec),1);
nascnum = zeros(floor(maxt/trec),1);
dgs = zeros(floor(maxt/trec),1);


% generate a list of possible events and their rates
% there will also be additional events that are added to the time lists

% stochastic events and their reference numbers
% 1 - switch in gene state (evind labels the state which is being switched
%     to)
% 2 - initiation of new polymerase
% 3 - decay of cytoplasmic RNA

% R2 = R;
% R2(logical(eye(size(R2)))) = 0;
% prevent staying in the same state being called an event

currtime = 0;
currnasc = 0;
currcyto = 0;
lastframe = 0; % the last frame recorded

% stochastic events

% deterministic events
dtime = [];
endflag = false;
% for this function we don't need to label the type or reference, because
% only 1 type of event can occur this way (RNA leaving the TS)
% dtype = [];
% dind = [];

% also, because at the moment, deterministic events are always added in
% order, no sorting is required; we can just use the first in-first out
% procedure.
while currtime<maxt
    evk = [];
    evtype = [];
    evind = [];
    
    % These could all be done at once, but it's less clear what's going on
    % then..
    % genestate transition
    evk = [evk;R(currstate,:)'];
    evtype = [evtype;ones(size(R,2),1)];
    evind = [evind;(1:size(R,2))'];
    
    % polymerase initiation
    evk = [evk;lambda(currstate)];
    evtype = [evtype;2];
    evind = [evind;NaN];
    
    % RNA decay
    evk = [evk;currcyto/tau];
    evtype = [evtype;3];
    evind = [evind;NaN];
    
    % Then generate an event time value, and see if it's smaller than the next
    % deterministic time.
    kcumu = cumsum(evk);
    evtime = -log(1-rand)/kcumu(end);
    
    ktime = currtime + evtime;
    
    if isempty(dtime) || ktime<dtime(1)
        currtime = ktime;
        kloop = true;
    else
        currtime = dtime(1);
        kloop = false;
    end
    
    % This updating should be done at the start! Because our event by
    % definition has occurred after these frames..
    % then check if any timepoints need recording
    if currtime>((lastframe+1)*trec)
        if currtime>maxt
            currtime = maxt;
            endflag = true;
        end
        recframes = (lastframe+1):floor(currtime/trec);
        dgs(recframes) = currstate;
        cytonum(recframes) = currcyto;
        nascnum(recframes) = currnasc;
        
        lastframe = recframes(end);
        
        if endflag
            break % with this here technically don't need any condition on the while loop..
        end
    end
    
    
    if kloop
        % implement the stochastic event and change the time to ktime
        
        currtime = ktime;
        
        evocc = nnz((kcumu(1:(end-1))/kcumu(end))<rand) + 1; 
        typocc = evtype(evocc);
        if typocc==1
            % genestate transition
            currstate = evind(evocc);
        elseif typocc==2
            % polymerase initiation
            % add a deterministic event to the end of the list
            dtime(end+1) = currtime + tdwell;
            
            currnasc = currnasc + 1;
        else
            % RNA decay
            
            currcyto = currcyto - 1;
        end
        
    else
        % implement the deterministic event and change the time to dtime
        currtime = dtime(1);
        dtime(1) = [];
        
        currnasc = currnasc - 1;
        currcyto = currcyto + 1;
    end
    
    
    
end


